package es.unican.docencia.y2012.ed.practica10;

/**
 * Clase que representa un aparato de frio/calor
 * @author Pablo Sanchez (p..sanchez@unican.es)
 * http://personales.unican.es/sanchezbp
 */
public class Heater extends Device implements ISuscriptor {
	
	protected boolean isActivated = false;
	// Inv: (0.0 <= temperature) && (temperature <= 50.0) 
	protected double temperature = 25.0;
	
	protected boolean isHeating = false;
	protected boolean isCooling = false;
	// Inv: ((isHeating)  || (isCooling)) implies isActivated
	// Inv: (isHeating) implies !(isCooling)
	// Inv: (isCooling) implies !(isHeating)
	
	/**
	 * Crear un nuevo aparato de frio/calor
	 * @param id El identificador para dicho aparato de frio/calor 
	 * @param room La habitacion donde se encuentra dicho aparato
	 */
	public Heater(String id, Room room) {
		super(id, room);
	} // Heater
	
	/**
	 * Activa el aparato de frio calor
	 */
	public void active() {
		this.isActivated = true;
	} // activate
	
	/**
	 * Desactiva el aparato de frio calor
	 */
	public void deactive() {
		this.isActivated = false;
	} // deactive
	
	/**
	 * Devuelve verdadero si el aparato esta activo	
	 * @return Verdadero si el aparato esta activo, falso 
	 * 		   en caso contrario
	 */
	public boolean isActive() {
		return isActivated;
	} // isCooling

	
	/**
	 * Devuelve verdadero si el aparato esta enfriando	
	 * @return Verdadero si el aparato esta enfriando, falso 
	 * 		   en caso contrario
	 */
	public boolean isCooling() {
		return isCooling;
	} // isCooling
	
	/**
	 * Devuelve verdadero si el aparato esta calentando	
	 * @return Verdadero si el aparato esta enfriando, falso 
	 * 		   en caso contrario
	 */
	public boolean isHeating() {
		return isHeating;
	} // isHeating
	
	/**
	 * Establece la temparatura del aparato de frio/calor
	 * @param temperature La temperatura ha ser asignada
	 * Pre: (0.0 <= temperature) && (temperature <= 50.0)
	 */
	public void setTemperature(double temperature) {
		this.temperature = temperature;
	} // setTemperature

	@Override
	public void deviceChanged(Device d) {
		assert (d instanceof Thermometer) && (d.getRoom().equals(this.room)) &&
			   (isActivated == true);
		
		Thermometer t = (Thermometer) d;
		
		
		if (t.getTemperature() == temperature) {
			isCooling = false;
			isHeating = false;
		} else if (t.getTemperature() < temperature) {
			this.isCooling = false;
			this.isHeating = true;
		} else {
			this.isHeating = false;
			this.isCooling = true;
		} // if
		
	} // deviceChanged
	
	@Override
	public String toString() {
		String out = "Heater "  + id + " temperature = " + temperature;
		if (!isActivated) {
			out += " is not active";
		} else if (isCooling) {
			out += " is cooling";
		} else if (isHeating){
			out += " is heating";
		} else {
			out += " is activated and idle"; 
		} // 
		return out;
	} // 
	
	
} // Heater
